#!/usr/local/bin/perl -wT

use CGI;
use CGI::Carp qw(fatalsToBrowser);
use HTML::Template;
use strict;

use vars qw(@names @suits @values @playercards @dealercards @deck
		$query $template $action $bank $bet $decks $playertotal
		$dealertotal $masterbet);


@names  = qw(Ace Two Three Four Five Six Seven Eight Nine Ten Jack Queen King);
@suits  = qw(Clubs Hearts Spades Diamonds);
@values = (1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 10, 10, 10);

$query = CGI->new();
$action = $query->param("action");


if($action eq "") {  # print out the bank/bet screen
 $template = HTML::Template->new(filename => "start.tmpl");
 print $query->header(), $template->output(), "\n";
 exit(0);
}

$bank        = $query->param("bank");
$bet         = $query->param("bet");
$masterbet   = $query->param("masterbet");
$decks       = $query->param("decks");
@deck        = $query->param("deck");

if($action eq "play") { # starting a new hand.
 if($decks > 12 || $decks < 1 || $bank < 0 || $bet < 0 || $bet > $bank) {
  print_template("badvalues.tmpl");
  exit(0);
 }

 # this is needed to restore the bet after doubling down
 $bet = $masterbet;

 # reshuffle half way through.
 if(@deck < $decks*26) { @deck = (0..$decks*52); }
 @playercards = (get_card(\@deck), get_card(\@deck));
 @dealercards = (get_card(\@deck));
 set_totals(); set_query_params();
 print_template("play.tmpl");
 exit(0);
}

@playercards = $query->param("playercards");
@dealercards = $query->param("dealercards");

if($action eq "hit") {
 push(@playercards, get_card(\@deck));
 set_totals(); set_query_params();
 if($playertotal > 21) {
  $bank -= $bet;
  print_template("playerbusted.tmpl");
  exit(0);
 }
 print_template("play.tmpl");
 exit(0);
}

if ($action eq "stand" || $action eq "double") {
 if($action eq "double") {
  $bet *= 2;
  push(@playercards, get_card(\@deck));
  set_totals(); set_query_params();
  if($playertotal > 21) {
   $bank -= $bet; set_query_params();
   print_template("playerbusted.tmpl");
   exit(0);
  }
 }

 set_totals();
 while($dealertotal < 17) {
  push(@dealercards, get_card(\@deck));
  set_totals();
 } 
 if($dealertotal > 21) {
  $bank += $bet; set_query_params();
  print_template("dealerbusted.tmpl");
 } elsif($dealertotal > $playertotal) {
  $bank -= $bet; set_query_params();
  print_template("dealerwon.tmpl");
 } elsif($dealertotal < $playertotal) {
  $bank += $bet; set_query_params();
  print_template("playerwon.tmpl");
 } else { 
  set_query_params();
  print_template("draw.tmpl");
 }
 exit(0);
}

die("unrecognized action");

use vars qw(@names @suits @values @playercards @dealercards @deck
		$query $template $action $bank $bet $decks $playertotal
		$dealertotal);

sub set_totals {
 $playertotal = total_hand(\@playercards);
 $dealertotal = total_hand(\@dealercards);
}

sub set_query_params {
 $query->delete_all();
 $query->param("playercards", @playercards);
 $query->param("dealercards", @dealercards);
 $query->param("deck",        @deck);
 $query->param("bank",        $bank);
 $query->param("bet",         $bet);
 $query->param("masterbet",   $masterbet);
 $query->param("decks",       $decks);
 $query->param("playertotal", $playertotal);
 $query->param("dealertotal", $dealertotal);
}

sub print_template {
 my $filename = shift;
 print $query->header(), read_template($filename), "\n";
}

sub read_template {
 my $filename = shift;
 my $template = HTML::Template->new(filename => $filename,
					die_on_bad_params => 0);
 my $hidden = '';
 my @temp;

 grep { 
	@temp = $query->param($_);
	if(@temp > 1) {
	 $template->param($_, [ map {{"value"=>$_};} @temp ]);
	} else {
         $template->param($_, $temp[0]);
	}
        $hidden .= $query->hidden($_); } $query->param();
 $template->param("hidden", $hidden);
 $template->param("playercards", [ map { { "value" => get_card_name($_) }; }
		$query->param("playercards") ] );
 $template->param("dealercards", [ map { { "value" => get_card_name($_) }; }
		$query->param("dealercards") ] );
 $template->param("url", $query->url());
 return $template->output();
}

sub total_hand {
 my $hand = shift;
 my $has_ace = 0;
 my $total = 0;

 foreach my $card( @$hand ) {
  $card %= 13;
  $total += $values[$card];
  if($values[$card] == 1) {
   $has_ace = 1;
  }
 }
 if($has_ace && $total <= 10) {
  $total += 10;
 }
 return $total;
}

sub get_card_name {
 return $names[$_[0]%13] . " of " . $suits[$_[0]%4];
}

sub get_card {
 my $deck = shift;
 my $offset = int(rand(@$deck));

 @$deck[$offset, $#$deck] = @$deck[$#$deck, $offset];
 return pop(@$deck);
}
